@@ -1,4 +1,4 @@
-package Pcore::API::Dynadot v0.3.2;
+package Pcore::API::Dynadot v0.3.3;
 
 use Pcore -dist, -class;
 use Pcore::API::Response;
@@ -29,7 +29,7 @@ sub check ( $self, $cb = undef ) {
     return $blocking_cv ? $blocking_cv->recv : ();
 }
 
-# NOTE max 100 domains are allowed
+# NOTE max 100 domains are allowed, 30 is recommneded
 sub search ( $self, $domains, $cb ) {
     die q[Max. 100 domains are allowed per search] if $domains->@* > 100;
 
@@ -50,12 +50,54 @@ sub search ( $self, $domains, $cb ) {
 
     my $domains_index = {};
 
-    for my $domain ( $domains->@* ) {
-        next if exists $domains_index->{$domain};
+    my $query_index;
 
-        $domains_index->{$domain} = undef;
+    for my $domain ( map { P->host($_) } $domains->@* ) {
+        my $domain_name = $domain->name;
 
-        push $url_params->@*, 'domain' . $i++, $domain;
+        next if exists $domains_index->{$domain_name};
+
+        $domains_index->{$domain_name} = undef;
+
+        if ( my $root_domain = $domain->root_domain ) {
+            my $dots = $root_domain =~ tr/././;
+
+            if ( !$dots ) {
+
+                # domain is TLD, is not available by default
+                $domains_index->{$domain_name} = 0;
+            }
+            elsif ( $dots > 2 ) {
+
+                # root domain contains > 3 labels, not available
+                $domains_index->{$domain_name} = 0;
+            }
+            else {
+
+                # root domain contains 2 (domain.TLD) or 3 (domain.pub_suffix.TLD) labels, can be checked
+                push $query_index->{$root_domain}->@*, $domain_name;
+
+                next if $query_index->{$root_domain}->@* > 1;
+
+                push $url_params->@*, 'domain' . $i++, $root_domain;
+            }
+        }
+
+        # no root domain, domain is pub. suffix, is not available by default
+        else {
+            $domains_index->{$domain_name} = 0;
+        }
+    }
+
+    # nothing to search
+    if ( !$query_index ) {
+        my $api_res = Pcore::API::Response->new( { status => 200 } );
+
+        $api_res->{result} = $domains_index;
+
+        $cb->($api_res);
+
+        return;
     }
 
     my $url = 'https://api.dynadot.com/api2.html?' . P->data->to_uri($url_params);
@@ -93,10 +135,10 @@ sub search ( $self, $domains, $cb ) {
                         my @fields = split /,/sm, $line;
 
                         if ( $fields[3] eq 'yes' ) {
-                            $domains_index->{ $fields[1] } = 1;
+                            $domains_index->@{ $query_index->{ $fields[1] }->@* } = (1) x $query_index->{ $fields[1] }->@*;
                         }
                         elsif ( $fields[3] eq 'no' ) {
-                            $domains_index->{ $fields[1] } = 0;
+                            $domains_index->@{ $query_index->{ $fields[1] }->@* } = (0) x $query_index->{ $fields[1] }->@*;
                         }
                     }
 
